# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import call_agent_call_agent_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.call_agent_call_agent_response import CallAgentCallAgentResponse

__all__ = ["CallAgentResource", "AsyncCallAgentResource"]


class CallAgentResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CallAgentResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Headless-Agents/headless-agents-python-sdk#accessing-raw-response-data-eg-headers
        """
        return CallAgentResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CallAgentResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Headless-Agents/headless-agents-python-sdk#with_streaming_response
        """
        return CallAgentResourceWithStreamingResponse(self)

    def call_agent(
        self,
        agent_id: str,
        *,
        conversation_id: str | NotGiven = NOT_GIVEN,
        request: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CallAgentCallAgentResponse:
        """
        Call an agent

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        return self._post(
            f"/call/{agent_id}",
            body=maybe_transform(
                {
                    "conversation_id": conversation_id,
                    "request": request,
                },
                call_agent_call_agent_params.CallAgentCallAgentParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CallAgentCallAgentResponse,
        )


class AsyncCallAgentResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCallAgentResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Headless-Agents/headless-agents-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncCallAgentResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCallAgentResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Headless-Agents/headless-agents-python-sdk#with_streaming_response
        """
        return AsyncCallAgentResourceWithStreamingResponse(self)

    async def call_agent(
        self,
        agent_id: str,
        *,
        conversation_id: str | NotGiven = NOT_GIVEN,
        request: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CallAgentCallAgentResponse:
        """
        Call an agent

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        return await self._post(
            f"/call/{agent_id}",
            body=await async_maybe_transform(
                {
                    "conversation_id": conversation_id,
                    "request": request,
                },
                call_agent_call_agent_params.CallAgentCallAgentParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CallAgentCallAgentResponse,
        )


class CallAgentResourceWithRawResponse:
    def __init__(self, call_agent: CallAgentResource) -> None:
        self._call_agent = call_agent

        self.call_agent = to_raw_response_wrapper(
            call_agent.call_agent,
        )


class AsyncCallAgentResourceWithRawResponse:
    def __init__(self, call_agent: AsyncCallAgentResource) -> None:
        self._call_agent = call_agent

        self.call_agent = async_to_raw_response_wrapper(
            call_agent.call_agent,
        )


class CallAgentResourceWithStreamingResponse:
    def __init__(self, call_agent: CallAgentResource) -> None:
        self._call_agent = call_agent

        self.call_agent = to_streamed_response_wrapper(
            call_agent.call_agent,
        )


class AsyncCallAgentResourceWithStreamingResponse:
    def __init__(self, call_agent: AsyncCallAgentResource) -> None:
        self._call_agent = call_agent

        self.call_agent = async_to_streamed_response_wrapper(
            call_agent.call_agent,
        )
