# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import create_schema_create_schema_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.create_schema_create_schema_response import CreateSchemaCreateSchemaResponse

__all__ = ["CreateSchemaResource", "AsyncCreateSchemaResource"]


class CreateSchemaResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CreateSchemaResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Headless-Agents/headless-agents-python-sdk#accessing-raw-response-data-eg-headers
        """
        return CreateSchemaResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CreateSchemaResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Headless-Agents/headless-agents-python-sdk#with_streaming_response
        """
        return CreateSchemaResourceWithStreamingResponse(self)

    def create_schema(
        self,
        *,
        prompt: str,
        conversation_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CreateSchemaCreateSchemaResponse:
        """
        Create a JSON schema based on user input

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/schema/create",
            body=maybe_transform(
                {
                    "prompt": prompt,
                    "conversation_id": conversation_id,
                },
                create_schema_create_schema_params.CreateSchemaCreateSchemaParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreateSchemaCreateSchemaResponse,
        )


class AsyncCreateSchemaResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCreateSchemaResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Headless-Agents/headless-agents-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncCreateSchemaResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCreateSchemaResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Headless-Agents/headless-agents-python-sdk#with_streaming_response
        """
        return AsyncCreateSchemaResourceWithStreamingResponse(self)

    async def create_schema(
        self,
        *,
        prompt: str,
        conversation_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CreateSchemaCreateSchemaResponse:
        """
        Create a JSON schema based on user input

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/schema/create",
            body=await async_maybe_transform(
                {
                    "prompt": prompt,
                    "conversation_id": conversation_id,
                },
                create_schema_create_schema_params.CreateSchemaCreateSchemaParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CreateSchemaCreateSchemaResponse,
        )


class CreateSchemaResourceWithRawResponse:
    def __init__(self, create_schema: CreateSchemaResource) -> None:
        self._create_schema = create_schema

        self.create_schema = to_raw_response_wrapper(
            create_schema.create_schema,
        )


class AsyncCreateSchemaResourceWithRawResponse:
    def __init__(self, create_schema: AsyncCreateSchemaResource) -> None:
        self._create_schema = create_schema

        self.create_schema = async_to_raw_response_wrapper(
            create_schema.create_schema,
        )


class CreateSchemaResourceWithStreamingResponse:
    def __init__(self, create_schema: CreateSchemaResource) -> None:
        self._create_schema = create_schema

        self.create_schema = to_streamed_response_wrapper(
            create_schema.create_schema,
        )


class AsyncCreateSchemaResourceWithStreamingResponse:
    def __init__(self, create_schema: AsyncCreateSchemaResource) -> None:
        self._create_schema = create_schema

        self.create_schema = async_to_streamed_response_wrapper(
            create_schema.create_schema,
        )
