# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List

import httpx

from ..types import process_objective_process_objective_params
from .._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from .._utils import (
    maybe_transform,
    async_maybe_transform,
)
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.process_objective_process_objective_response import ProcessObjectiveProcessObjectiveResponse

__all__ = ["ProcessObjectiveResource", "AsyncProcessObjectiveResource"]


class ProcessObjectiveResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ProcessObjectiveResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Headless-Agents/headless-agents-python-sdk#accessing-raw-response-data-eg-headers
        """
        return ProcessObjectiveResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ProcessObjectiveResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Headless-Agents/headless-agents-python-sdk#with_streaming_response
        """
        return ProcessObjectiveResourceWithStreamingResponse(self)

    def process_objective(
        self,
        *,
        agent_ids: List[str],
        user_objective: str,
        conversation_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ProcessObjectiveProcessObjectiveResponse:
        """
        Process user objective against multiple agents

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/digest",
            body=maybe_transform(
                {
                    "agent_ids": agent_ids,
                    "user_objective": user_objective,
                    "conversation_id": conversation_id,
                },
                process_objective_process_objective_params.ProcessObjectiveProcessObjectiveParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProcessObjectiveProcessObjectiveResponse,
        )


class AsyncProcessObjectiveResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncProcessObjectiveResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Headless-Agents/headless-agents-python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncProcessObjectiveResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncProcessObjectiveResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Headless-Agents/headless-agents-python-sdk#with_streaming_response
        """
        return AsyncProcessObjectiveResourceWithStreamingResponse(self)

    async def process_objective(
        self,
        *,
        agent_ids: List[str],
        user_objective: str,
        conversation_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ProcessObjectiveProcessObjectiveResponse:
        """
        Process user objective against multiple agents

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/digest",
            body=await async_maybe_transform(
                {
                    "agent_ids": agent_ids,
                    "user_objective": user_objective,
                    "conversation_id": conversation_id,
                },
                process_objective_process_objective_params.ProcessObjectiveProcessObjectiveParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ProcessObjectiveProcessObjectiveResponse,
        )


class ProcessObjectiveResourceWithRawResponse:
    def __init__(self, process_objective: ProcessObjectiveResource) -> None:
        self._process_objective = process_objective

        self.process_objective = to_raw_response_wrapper(
            process_objective.process_objective,
        )


class AsyncProcessObjectiveResourceWithRawResponse:
    def __init__(self, process_objective: AsyncProcessObjectiveResource) -> None:
        self._process_objective = process_objective

        self.process_objective = async_to_raw_response_wrapper(
            process_objective.process_objective,
        )


class ProcessObjectiveResourceWithStreamingResponse:
    def __init__(self, process_objective: ProcessObjectiveResource) -> None:
        self._process_objective = process_objective

        self.process_objective = to_streamed_response_wrapper(
            process_objective.process_objective,
        )


class AsyncProcessObjectiveResourceWithStreamingResponse:
    def __init__(self, process_objective: AsyncProcessObjectiveResource) -> None:
        self._process_objective = process_objective

        self.process_objective = async_to_streamed_response_wrapper(
            process_objective.process_objective,
        )
